@@ -69,6 +69,7 @@ struct _CountdownHomeWidgetPrivate
 	guint bgOpacity, buttonAreaOpacity;
 	gchar *bgColor, *buttonAreaColor;
 	guint radius;
+	guint displaySize;
 	gchar *yearsText, *weeksText, *daysText;
 	gchar *minutesText, *hoursText;
 	gchar *leftText, *agoText;
@@ -157,6 +158,7 @@ void countdown_home_widget_write_settings (CountdownHomeWidget *self)
 	g_key_file_set_string (keyFile, "config", "titleFont", self->priv->titleFont);
 	g_key_file_set_string (keyFile, "config", "valueColor", self->priv->valueColor);
 	g_key_file_set_string (keyFile, "config", "valueFont", self->priv->valueFont);
+	g_key_file_set_integer (keyFile, "config", "displaySize", self->priv->displaySize);
 	g_key_file_set_string (keyFile, "labels", "yearsText", self->priv->yearsText);
 	g_key_file_set_string (keyFile, "labels", "weeksText", self->priv->weeksText);
 	g_key_file_set_string (keyFile, "labels", "daysText", self->priv->daysText);
@@ -667,7 +669,11 @@ void countdown_home_widget_global_options (HDHomePluginItem *hitem, CountdownHom
 	bgDialog = gtk_dialog_new_with_buttons (_("Background Settings"), GTK_WINDOW (dialog),
 		GTK_DIALOG_DESTROY_WITH_PARENT, _tran("wdgt_bd_done"), GTK_RESPONSE_ACCEPT, NULL);
 	bgContentArea = gtk_dialog_get_content_area (GTK_DIALOG (bgDialog));
+	GtkWidget *scrollBG = hildon_pannable_area_new ();
 	GtkWidget *bgBox = gtk_vbox_new (FALSE, 0);
+	hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (scrollBG), bgBox);
+	gtk_container_add (GTK_CONTAINER (bgContentArea), scrollBG);
+	gtk_window_set_default_size (GTK_WINDOW (bgDialog), -1, 350);
 	
 	GtkWidget *bgSystemStyle = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
 	gtk_button_set_label (GTK_BUTTON (bgSystemStyle), _("Use system style"));
@@ -676,7 +682,7 @@ void countdown_home_widget_global_options (HDHomePluginItem *hitem, CountdownHom
 	g_signal_connect (bgSystemStyle, "toggled", G_CALLBACK (countdown_home_widget_system_settings), bgSettingsButton);
 	GtkWidget *bgSystemStyleCaption = hildon_caption_new (bgGroup, "", bgSystemStyle, NULL, HILDON_CAPTION_MANDATORY);
 	hildon_caption_set_separator (HILDON_CAPTION (bgSystemStyleCaption), "");
-	gtk_container_add (GTK_CONTAINER (bgContentArea), bgSystemStyleCaption);	
+	gtk_container_add (GTK_CONTAINER (bgBox), bgSystemStyleCaption);	
 	
 	GtkWidget *bgColorButton = he_color_button_new ();
 	hildon_gtk_widget_set_theme_size (bgColorButton, HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
@@ -730,7 +736,21 @@ void countdown_home_widget_global_options (HDHomePluginItem *hitem, CountdownHom
 	hildon_button_add_title_size_group (HILDON_BUTTON (radiusButton), bgGroup);
 	gtk_container_add (GTK_CONTAINER (bgBox), radiusButton);
 	
-	gtk_container_add (GTK_CONTAINER (bgContentArea), bgBox);
+	GtkWidget *displaySizeSelector = hildon_touch_selector_new_text ();
+	for (i=0; i<=4; i++) {
+		gchar *text = g_strdup_printf ("%d", i + 1);
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (displaySizeSelector), text);
+		g_free (text);
+	}
+	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (displaySizeSelector), 0, self->priv->displaySize);
+	GtkWidget *displaySizeButton = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
+	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (displaySizeButton), HILDON_TOUCH_SELECTOR (displaySizeSelector));
+	hildon_button_set_title (HILDON_BUTTON (displaySizeButton), _("Display size"));
+	hildon_button_set_alignment (HILDON_BUTTON (displaySizeButton), 0, 0.5, 0, 0);
+	hildon_button_add_title_size_group (HILDON_BUTTON (displaySizeButton), bgGroup);
+	gtk_container_add (GTK_CONTAINER (bgBox), displaySizeButton);
+	
+	//gtk_container_add (GTK_CONTAINER (bgContentArea), bgBox);
 	
 	transDialog = gtk_dialog_new_with_buttons (_("Change how terms display"), GTK_WINDOW (dialog),
 		GTK_DIALOG_DESTROY_WITH_PARENT, _tran("wdgt_bd_done"), GTK_RESPONSE_ACCEPT, NULL);
@@ -854,6 +874,7 @@ void countdown_home_widget_global_options (HDHomePluginItem *hitem, CountdownHom
 		self->priv->bgColor = gdk_color_to_string (&color);
 		self->priv->bgOpacity = gtk_range_get_value (GTK_RANGE (bgOpacity));
 		self->priv->radius = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (radiusSelector), 0);
+		self->priv->displaySize = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (displaySizeSelector), 0);
 		
 		he_color_button_get_color (HE_COLOR_BUTTON (buttonAreaColorButton), &color);
 		self->priv->buttonAreaColor = gdk_color_to_string (&color);
@@ -873,6 +894,15 @@ void countdown_home_widget_global_options (HDHomePluginItem *hitem, CountdownHom
 				self->priv->eventID = g_timeout_add (self->priv->refresh*1000*60, (GSourceFunc)countdown_home_widget_update_content, self);
 			}
 		}
+	
+
+    		if (!self->priv->systemBG) {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 312, (self->priv->displaySize + 1) * 64 + 48);
+			gtk_window_resize (GTK_WINDOW (self), 312, (self->priv->displaySize + 1) * 64 +48);
+		} else {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 312, 368);
+			gtk_window_resize (GTK_WINDOW (self), 312, 368);
+		}
 		
 		countdown_home_widget_write_settings (self);
 		countdown_home_widget_update_content (self);
@@ -993,6 +1023,12 @@ void countdown_home_widget_make_view ( CountdownHomeWidget *self )
 			g_error_free (error);
 			error = NULL;
 		}
+		self->priv->displaySize = g_key_file_get_integer (keyFile, "config", "displaySize", &error);
+		if (error) {
+			self->priv->displaySize = 5;
+			g_error_free (error);
+			error = NULL;
+		}
 		self->priv->buttonAreaColor = g_key_file_get_string (keyFile, "config", "buttonAreaColor", &error);
 		if (error) {
 			self->priv->buttonAreaColor = "#000000";
@@ -1084,6 +1120,7 @@ void countdown_home_widget_make_view ( CountdownHomeWidget *self )
 		self->priv->bgOpacity = 50;
 		self->priv->bgColor = "#000000";
 		self->priv->radius = 10;
+		self->priv->displaySize = 5;
 		self->priv->buttonAreaOpacity = 75;
 		self->priv->buttonAreaColor = "#000000";
 		self->priv->systemFont = TRUE;
@@ -1100,6 +1137,11 @@ void countdown_home_widget_make_view ( CountdownHomeWidget *self )
 		self->priv->agoText = _("ago");
     }
 
+    if (!self->priv->systemBG) {
+	gtk_widget_set_size_request (GTK_WIDGET (self), 312, (self->priv->displaySize + 1) * 64 + 48);
+	gtk_window_resize (GTK_WINDOW (self), 312, (self->priv->displaySize + 1) * 64 + 48);
+    }
+    
     g_key_file_free (keyFile);
     g_free (filename);
 }
@@ -1269,10 +1311,15 @@ gboolean countdown_home_widget_update_content (CountdownHomeWidget *self)
 	}
 
 	totalRows=gtk_tree_model_iter_n_children (sortModel, NULL);
-	if (totalRows <= ENTRIES_TO_DISPLAY || self->priv->offset < 0) {
+	int totalNumber = ENTRIES_TO_DISPLAY;
+	if (!self->priv->systemBG) {
+		totalNumber = self->priv->displaySize + 1;
+	}
+	
+	if (totalRows <= totalNumber || self->priv->offset < 0) {
 		self->priv->offset = 0;
-	} else if (totalRows > ENTRIES_TO_DISPLAY && self->priv->offset > totalRows - ENTRIES_TO_DISPLAY) {
-		self->priv->offset = totalRows - ENTRIES_TO_DISPLAY;
+	} else if (totalRows > totalNumber && self->priv->offset > totalRows - totalNumber) {
+		self->priv->offset = totalRows - totalNumber;
 	}
 	
 	gtk_container_foreach (GTK_CONTAINER (self->priv->contents), (GtkCallback) gtk_widget_destroy, NULL);
@@ -1435,7 +1482,7 @@ gboolean countdown_home_widget_update_content (CountdownHomeWidget *self)
 			
 			exists = gtk_tree_model_iter_next (sortModel, &iter);
 			if (exists) {
-				if (count >= ENTRIES_TO_DISPLAY) {
+				if (count >= totalNumber) {
 					exists = FALSE;
 				}
 			}
@@ -1447,14 +1494,14 @@ gboolean countdown_home_widget_update_content (CountdownHomeWidget *self)
 		gtk_container_add (GTK_CONTAINER (self->priv->contents), label);
 	}
 
-	if (totalRows > ENTRIES_TO_DISPLAY) {
+	if (totalRows > totalNumber) {
 		if (self->priv->offset == 0) {
 			gtk_widget_set_sensitive (self->priv->upButton, FALSE);
 		} else {
 			gtk_widget_set_sensitive (self->priv->upButton, TRUE);
 		}
 		
-		if (self->priv->offset + ENTRIES_TO_DISPLAY >= totalRows) {
+		if (self->priv->offset + totalNumber >= totalRows) {
 			gtk_widget_set_sensitive (self->priv->downButton, FALSE);
 		} else {
 			gtk_widget_set_sensitive (self->priv->downButton, TRUE);
@@ -1767,7 +1814,7 @@ static void countdown_home_widget_init (CountdownHomeWidget *self)
 
 	GdkGeometry hints;
 	hints.min_width = 316;
-	hints.min_height = 368;
+	hints.min_height = 112;
 	hints.max_width = 316;
 	hints.max_height = 368;
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
